@@ -226,6 +226,7 @@ typedef enum {
   DT_H5LT_lang_t,
   DT_H5O_mcdt_search_ret_t,
   DT_H5O_type_t,
+  DT_H5PL_type_t,
   DT_H5R_type_t,
   DT_H5S_class_t,
   DT_H5S_sel_type,
@@ -317,7 +318,7 @@ hid_t create_DT_H5D_mpio_actual_chunk_opt_mode_t(void);
 /* typedef enum H5D_mpio_actual_io_mode_t { H5D_MPIO_NO_COLLECTIVE = 0x0, H5D_MPIO_CHUNK_INDEPENDENT = 0x1, H5D_MPIO_CHUNK_COLLECTIVE = 0x2, H5D_MPIO_CHUNK_MIXED = 0x1 | 0x2, H5D_MPIO_CONTIGUOUS_COLLECTIVE = 0x4} H5D_mpio_actual_io_mode_t;  */
 hid_t create_DT_H5D_mpio_actual_io_mode_t(void);
 
-/* typedef enum H5D_mpio_no_collective_cause_t { H5D_MPIO_COLLECTIVE = 0x00, H5D_MPIO_SET_INDEPENDENT = 0x01, H5D_MPIO_DATATYPE_CONVERSION = 0x02, H5D_MPIO_DATA_TRANSFORMS = 0x04, H5D_MPIO_SET_MPIPOSIX = 0x08, H5D_MPIO_NOT_SIMPLE_OR_SCALAR_DATASPACES = 0x10, H5D_MPIO_POINT_SELECTIONS = 0x20, H5D_MPIO_NOT_CONTIGUOUS_OR_CHUNKED_DATASET = 0x40, H5D_MPIO_FILTERS = 0x80} H5D_mpio_no_collective_cause_t; */
+/* typedef enum H5D_mpio_no_collective_cause_t { H5D_MPIO_COLLECTIVE = 0x00, H5D_MPIO_SET_INDEPENDENT = 0x01, H5D_MPIO_DATATYPE_CONVERSION = 0x02, H5D_MPIO_DATA_TRANSFORMS = 0x04, H5D_MPIO_MPI_OPT_TYPES_ENV_VAR_DISABLED = 0x08, H5D_MPIO_NOT_SIMPLE_OR_SCALAR_DATASPACES = 0x10, H5D_MPIO_NOT_CONTIGUOUS_OR_CHUNKED_DATASET = 0x20, H5D_MPIO_FILTERS = 0x40} H5D_mpio_no_collective_cause_t; */
 hid_t create_DT_H5D_mpio_no_collective_cause_t(void);
 
 /* typedef enum H5D_space_status_t { H5D_SPACE_STATUS_ERROR = -1, H5D_SPACE_STATUS_NOT_ALLOCATED = 0, H5D_SPACE_STATUS_PART_ALLOCATED = 1, H5D_SPACE_STATUS_ALLOCATED = 2} H5D_space_status_t; */
@@ -365,6 +366,9 @@ hid_t create_DT_H5O_mcdt_search_ret_t(void);
 /* typedef enum H5O_type_t { H5O_TYPE_UNKNOWN = -1, H5O_TYPE_GROUP, H5O_TYPE_DATASET, H5O_TYPE_NAMED_DATATYPE, H5O_TYPE_NTYPES } H5O_type_t; */
 hid_t create_DT_H5O_type_t(void);
 
+/* typedef enum H5PL_type_t { H5PL_TYPE_ERROR = -1, H5PL_TYPE_FILTER = 0, H5PL_TYPE_NONE = 1 } H5PL_type_t; */
+hid_t create_DT_H5PL_type_t(void);
+
 /* typedef enum { H5R_BADTYPE = (-1), H5R_OBJECT, H5R_DATASET_REGION, H5R_MAXTYPE } H5R_type_t; */
 hid_t create_DT_H5R_type_t(void);
 
